<%@ Page Language="C#" Trace="false" %>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<%@ Import Namespace="System.Security.Cryptography.X509Certificates" %>
<%@ Import Namespace="System.Security.Cryptography" %>


<script runat="server" language="c#">
    string error = String.Empty;
    string message = String.Empty;
    string warning = String.Empty;
    protected void btnConfirm_Click(object sender, EventArgs e)
    {
        if (!txtIAgree.Checked)
        {
            error = "You didn't confirm that this certificate has been assigned to you";
            return;
        }
        if (!Page.Request.ClientCertificate.IsPresent)
        {
            error = "No certificate found";
            return;
        }
        if (!User.Identity.IsAuthenticated)
        {
            error = "The current user is not authenticated";
            return;
        }
        txtIAgree.Checked = false;
        try
        {
            LinkCertificate();
            message = "The certificate has been successfully collected";
        }
        catch(Exception ex)
        {
            warning = "An error occured: <br>";
            warning += ex.Message + "<br>";
            warning += ex.StackTrace.Replace(System.Environment.NewLine, "<br>");
        }
    }
    void LoadCertInfo()
    {
        string para = "<div style='margin: 10px 0 0 0; font-weight: bold'>{0}</div>";
        string subpara = "<div style='margin-left: 15px; font-size: 90%'>{0}</div>";
    
        if (Page.Request.ClientCertificate.IsPresent)
        {
            Response.Write("<div style='margin: 20px'>");
            try
            {
                X509Certificate2 x509Cert2 = new X509Certificate2(Page.Request.ClientCertificate.Certificate);

                Response.Write(string.Format(para, "Issued To:"));
                Response.Write(string.Format(subpara, x509Cert2.Subject));

                Response.Write(string.Format(para, "Issued By:"));
                Response.Write(string.Format(subpara, x509Cert2.Issuer));

                Response.Write(string.Format(para, "Friendly Name:"));
                Response.Write(string.Format(subpara, string.IsNullOrEmpty(x509Cert2.FriendlyName) ? "(None Specified)" : x509Cert2.FriendlyName));

                Response.Write(string.Format(para, "Valid Dates:"));
                Response.Write(string.Format(subpara, "From: " + x509Cert2.GetEffectiveDateString()));
                Response.Write(string.Format(subpara, "To: " + x509Cert2.GetExpirationDateString()));

                Response.Write(string.Format(para, "Thumbprint:"));
                Response.Write(string.Format(subpara, x509Cert2.Thumbprint));

                //Response.Write(string.Format(para, "Public Key:"));
                //Response.Write(string.Format(subpara, x509Cert2.GetPublicKeyString()));

                #region EKU Section - Retrieve EKU info and write out each OID
                X509EnhancedKeyUsageExtension ekuExtension = (X509EnhancedKeyUsageExtension)x509Cert2.Extensions["Enhanced Key Usage"];
                if (ekuExtension != null)
                {
                    Response.Write(string.Format(para, "Enhanced Key Usages (" + ekuExtension.EnhancedKeyUsages.Count.ToString() + " found)"));

                    OidCollection ekuOids = ekuExtension.EnhancedKeyUsages;
                    foreach (Oid ekuOid in ekuOids)
                        Response.Write(string.Format(subpara, ekuOid.FriendlyName + " (OID: " + ekuOid.Value + ")"));
                }
                else
                {
                    Response.Write(string.Format(para, "No EKU Section Data"));
                }
                #endregion // EKU Section

                #region Subject Alternative Name Section
                X509Extension sanExtension = (X509Extension)x509Cert2.Extensions["Subject Alternative Name"];
                if (sanExtension != null)
                {
                    Response.Write(string.Format(para, "Subject Alternative Name:"));
                    Response.Write(string.Format(subpara, sanExtension.Format(true)));
                }
                else
                {
                    Response.Write(string.Format(para, "No Subject Alternative Name Data"));
                }

                #endregion // Subject Alternative Name Section

                #region Certificate Policies Section
                X509Extension policyExtension = (X509Extension)x509Cert2.Extensions["Certificate Policies"];
                if (policyExtension != null)
                        {
                            Response.Write(string.Format(para, "Certificate Policies:"));
                            Response.Write(string.Format(subpara, policyExtension.Format(true)));
                        }
                        else
                        {
                            Response.Write(string.Format(para, "No Certificate Policies Data"));
                        }
                #endregion //Certificate Policies Section


                // Example on how to enumerate all extensions
                //foreach (X509Extension extension in x509Cert2.Extensions)
                //    Response.Write(string.Format(para, extension.Oid.FriendlyName + "(" + extension.Oid.Value + ")"));
            }
            catch (Exception ex)
            {
                Response.Write(string.Format(para, "An error occured:"));
                Response.Write(string.Format(subpara, ex.Message));
                Response.Write(string.Format(subpara, ex.StackTrace));
            }
            finally
            {
                Response.Write("</div>");
            }
        }
        else
        {
            Response.Write("no certificate detected");
        }
    }
    void LinkCertificate()
    {
        string user = HttpContext.Current.User.Identity.Name;
        byte[] certificate = Page.Request.ClientCertificate.Certificate;
	string filename = Server.MapPath("certificates") + "/" +  Convert.ToString(System.DateTime.Now.ToFileTimeUtc(),16) +"-" + user.Replace("\\", "-") + ".cer";
	System.IO.File.WriteAllBytes(filename, certificate);
    }
</script>

<html>
<head runat="server">
    <title>Collect a certificate</title>
    <style media="screen" type="text/css">
        body {
            font-family: Arial, Helvetica, sans-serif;
            font-size: 13px;
        }

        label {
            width: 150px; /*Or however much space you need for the forms labels*/
            float: left;
        }

        .info, .success, .warning, .error, .validation {
            border: 1px solid;
            margin: 10px 0px;
            padding: 15px 10px 15px 50px;
            background-repeat: no-repeat;
            background-position: 10px center;
        }

        .info {
            color: #00529B;
            background-color: #BDE5F8;
            background-image: url('info.png');
        }

        .success {
            color: #4F8A10;
            background-color: #DFF2BF;
            background-image: url('success.png');
        }

        .warning {
            color: #9F6000;
            background-color: #FEEFB3;
            background-image: url('warning.png');
        }

        .error {
            color: #D8000C;
            background-color: #FFBABA;
            background-image: url('error.png');
        }
    </style>
</head>
<body>
    <img src="logo.jpg" />

    <h1>Collect a certificate</h1>
    <p>
        This page collects the certificate to be linked to your user account. 
Once the administrator approves the association, 
you'll be able to open a Windows session with this certificate.
    </p>

    <% if (!String.IsNullOrEmpty(error)) {%>
    <p class='error'><%=error%></p>
    <%}%>

    <% if (!String.IsNullOrEmpty(warning)) {%>
    <p class='warning'><%=warning%></p>
    <%}%>

    <% if (!String.IsNullOrEmpty(message)) {%>
    <p class='success'><%=message%></p>
    <%}%>

    <h2>Your identity</h2>
    You are connected as <% =HttpContext.Current.User.Identity.Name%>.

    <h2>Your certificate</h2>
    <% LoadCertInfo();  %>

    <h2>Confirmation</h2>
    <div>
        <form runat="server">
            <fieldset>

                <table>
                    <tr>
                        <td width="100%" style="white-space: nowrap; overflow: hidden;">
                            <asp:checkbox id="txtIAgree" runat="server" checked="false" />
                            I confirm that the certificate I'm asking to configure has been assigned to me.
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <asp:button id="btnConfirm" runat="server" onclick="btnConfirm_Click" text="Confirm" />
                        </td>
                    </tr>
                </table>
            </fieldset>
        </form>
    </div>
</body>
</html>
